package com.sun.org.apache.bcel.internal.generic;

import com.sun.org.apache.bcel.internal.classfile.LocalVariable;

public class LocalVariableGen implements InstructionTargeter, NamedAndTyped, Cloneable
{
    private int index;
    private String name;
    private Type type;
    private InstructionHandle start;
    private InstructionHandle end;
    private int orig_index;
    private boolean live_to_end;
    
    public LocalVariableGen(int index, String name, Type type, InstructionHandle start, InstructionHandle end) {
        if (index < 0 || index > 65535) {
            throw new ClassGenException("Invalid index index: " + index);
        }
        this.name = name;
        this.type = type;
        this.index = index;
        this.setStart(start);
        this.setEnd(end);
        this.orig_index = index;
        this.live_to_end = (end == null);
        return;
    }
    
    public LocalVariableGen(int index, String name, Type type, InstructionHandle start, InstructionHandle end, int orig_index) {
        this(index, name, type, start, end);
        this.orig_index = orig_index;
    }
    
    public LocalVariable getLocalVariable(ConstantPoolGen cp) {
        int start_pc = 0;
        int length = 0;
        if (this.start != null && this.end != null) {
            start_pc = this.start.getPosition();
            length = this.end.getPosition() - start_pc;
            if (this.end.getNext() == null && this.live_to_end) {
                length += this.end.getInstruction().getLength();
            }
        }
        int name_index = cp.addUtf8(this.name);
        int signature_index = cp.addUtf8(this.type.getSignature());
        return new LocalVariable(start_pc, length, name_index, signature_index, this.index, cp.getConstantPool(), this.orig_index);
    }
    
    public void setIndex(int index) {
        this.index = index;
    }
    
    public int getIndex() {
        return this.index;
    }
    
    public int getOrigIndex() {
        return this.orig_index;
    }
    
    public void setLiveToEnd(boolean live_to_end) {
        this.live_to_end = live_to_end;
    }
    
    public boolean getLiveToEnd() {
        return this.live_to_end;
    }
    
    @Override
    public void setName(String name) {
        this.name = name;
    }
    
    @Override
    public String getName() {
        return this.name;
    }
    
    @Override
    public void setType(Type type) {
        this.type = type;
    }
    
    @Override
    public Type getType() {
        return this.type;
    }
    
    public InstructionHandle getStart() {
        return this.start;
    }
    
    public InstructionHandle getEnd() {
        return this.end;
    }
    
    public void setStart(InstructionHandle start) {
        BranchInstruction.notifyTarget(this.start, start, this);
        this.start = start;
    }
    
    public void setEnd(InstructionHandle end) {
        BranchInstruction.notifyTarget(this.end, end, this);
        this.end = end;
    }
    
    @Override
    public void updateTarget(InstructionHandle old_ih, InstructionHandle new_ih) {
        boolean targeted = false;
        if (this.start == old_ih) {
            targeted = true;
            this.setStart(new_ih);
        }
        if (this.end == old_ih) {
            targeted = true;
            this.setEnd(new_ih);
        }
        if (!targeted) {
            throw new ClassGenException("Not targeting " + old_ih + ", but {" + this.start + ", " + this.end + "}");
        }
        return;
    }
    
    void dispose() {
        this.setStart(null);
        this.setEnd(null);
    }
    
    @Override
    public boolean containsTarget(InstructionHandle ih) {
        return this.start == ih || this.end == ih;
    }
    
    @Override
    public int hashCode() {
        return this.name.hashCode() ^ this.type.hashCode();
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof LocalVariableGen)) {
            return false;
        }
        LocalVariableGen l = (LocalVariableGen)o;
        return l.index == this.index && l.start == this.start && l.end == this.end;
    }
    
    @Override
    public String toString() {
        return "LocalVariableGen(" + this.name + ", " + this.type + ", " + this.start + ", " + this.end + ")";
    }
    
    public Object clone() {
        try {
            return super.clone();
        }
        catch (CloneNotSupportedException e) {
            throw new Error("Clone Not Supported");
        }
    }
}
